@@ -3,12 +3,12 @@
  * parse_clause.c
  *	  handle clauses in parser
  *
- * Portions Copyright (c) 1996-2002, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2003, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $Header: /cvsroot/pgsql/src/backend/parser/parse_clause.c,v 1.98.2.1 2002/12/16 18:39:56 tgl Exp $
+ *	  $Header: /cvsroot/pgsql-server/src/backend/parser/parse_clause.c,v 1.124.2.1 2004/04/18 18:13:31 tgl Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -22,7 +22,6 @@
 #include "optimizer/tlist.h"
 #include "optimizer/var.h"
 #include "parser/analyze.h"
-#include "parser/parse.h"
 #include "parser/parsetree.h"
 #include "parser/parse_clause.h"
 #include "parser/parse_coerce.h"
@@ -31,6 +30,7 @@
 #include "parser/parse_relation.h"
 #include "parser/parse_target.h"
 #include "parser/parse_type.h"
+#include "rewrite/rewriteManip.h"
 #include "utils/builtins.h"
 #include "utils/guc.h"
 
@@ -55,13 +55,10 @@ static RangeTblRef *transformRangeFunction(ParseState *pstate,
 					   RangeFunction *r);
 static Node *transformFromClauseItem(ParseState *pstate, Node *n,
 						List **containedRels);
-static Node *buildMergedJoinVar(JoinType jointype,
+static Node *buildMergedJoinVar(ParseState *pstate, JoinType jointype,
 				   Var *l_colvar, Var *r_colvar);
 static TargetEntry *findTargetlistEntry(ParseState *pstate, Node *node,
 					List *tlist, int clause);
-static List *addTargetToSortList(TargetEntry *tle, List *sortlist,
-					List *targetlist, List *opname);
-static bool targetIsInSortList(TargetEntry *tle, List *sortList);
 
 
 /*
@@ -192,7 +189,7 @@ interpretInhOption(InhOption inhOpt)
 		case INH_DEFAULT:
 			return SQL_inheritance;
 	}
-	elog(ERROR, "Bogus InhOption value");
+	elog(ERROR, "bogus InhOption value");
 	return false;				/* keep compiler quiet */
 }
 
@@ -263,7 +260,7 @@ transformJoinUsingClause(ParseState *pstate, List *leftVars, List *rightVars)
 		Node	   *rvar = (Node *) lfirst(rvars);
 		A_Expr	   *e;
 
-		e = makeSimpleA_Expr(OP, "=", copyObject(lvar), copyObject(rvar));
+		e = makeSimpleA_Expr(AEXPR_OP, "=", copyObject(lvar), copyObject(rvar));
 
 		if (result == NULL)
 			result = (Node *) e;
@@ -271,7 +268,7 @@ transformJoinUsingClause(ParseState *pstate, List *leftVars, List *rightVars)
 		{
 			A_Expr	   *a;
 
-			a = makeA_Expr(AND, NIL, result, (Node *) e);
+			a = makeA_Expr(AEXPR_AND, NIL, result, (Node *) e);
 			result = (Node *) a;
 		}
 
@@ -286,7 +283,7 @@ transformJoinUsingClause(ParseState *pstate, List *leftVars, List *rightVars)
 	 */
 	result = transformExpr(pstate, result);
 
-	result = coerce_to_boolean(result, "JOIN/USING");
+	result = coerce_to_boolean(pstate, result, "JOIN/USING");
 
 	return result;
 }	/* transformJoinUsingClause() */
@@ -301,8 +298,8 @@ transformJoinOnClause(ParseState *pstate, JoinExpr *j,
 {
 	Node	   *result;
 	List	   *save_namespace;
-	List	   *clause_varnos,
-			   *l;
+	Relids		clause_varnos;
+	int			varno;
 
 	/*
 	 * This is a tad tricky, for two reasons.  First, the namespace that
@@ -317,10 +314,7 @@ transformJoinOnClause(ParseState *pstate, JoinExpr *j,
 	save_namespace = pstate->p_namespace;
 	pstate->p_namespace = makeList2(j->larg, j->rarg);
 
-	/* This part is just like transformWhereClause() */
-	result = transformExpr(pstate, j->quals);
-
-	result = coerce_to_boolean(result, "JOIN/ON");
+	result = transformWhereClause(pstate, j->quals, "JOIN/ON");
 
 	pstate->p_namespace = save_namespace;
 
@@ -333,17 +327,17 @@ transformJoinOnClause(ParseState *pstate, JoinExpr *j,
 	 * here.)
 	 */
 	clause_varnos = pull_varnos(result);
-	foreach(l, clause_varnos)
+	while ((varno = bms_first_member(clause_varnos)) >= 0)
 	{
-		int			varno = lfirsti(l);
-
 		if (!intMember(varno, containedRels))
 		{
-			elog(ERROR, "JOIN/ON clause refers to \"%s\", which is not part of JOIN",
-				 rt_fetch(varno, pstate->p_rtable)->eref->aliasname);
+			ereport(ERROR,
+					(errcode(ERRCODE_INVALID_COLUMN_REFERENCE),
+					 errmsg("JOIN/ON clause refers to \"%s\", which is not part of JOIN",
+				   rt_fetch(varno, pstate->p_rtable)->eref->aliasname)));
 		}
 	}
-	freeList(clause_varnos);
+	bms_free(clause_varnos);
 
 	return result;
 }
@@ -386,7 +380,6 @@ transformTableEntry(ParseState *pstate, RangeVar *r)
 static RangeTblRef *
 transformRangeSubselect(ParseState *pstate, RangeSubselect *r)
 {
-	List	   *save_namespace;
 	List	   *parsetrees;
 	Query	   *query;
 	RangeTblEntry *rte;
@@ -398,39 +391,55 @@ transformRangeSubselect(ParseState *pstate, RangeSubselect *r)
 	 * an unlabeled subselect.
 	 */
 	if (r->alias == NULL)
-		elog(ERROR, "sub-select in FROM must have an alias");
+		ereport(ERROR,
+				(errcode(ERRCODE_SYNTAX_ERROR),
+				 errmsg("subquery in FROM must have an alias")));
 
 	/*
-	 * Analyze and transform the subquery.	This is a bit tricky because
-	 * we don't want the subquery to be able to see any FROM items already
-	 * created in the current query (per SQL92, the scope of a FROM item
-	 * does not include other FROM items).	But it does need to be able to
-	 * see any further-up parent states, so we can't just pass a null
-	 * parent pstate link.	So, temporarily make the current query level
-	 * have an empty namespace.
+	 * Analyze and transform the subquery.
 	 */
-	save_namespace = pstate->p_namespace;
-	pstate->p_namespace = NIL;
-
-	parsetrees = parse_analyze(r->subquery, pstate);
-
-	pstate->p_namespace = save_namespace;
+	parsetrees = parse_sub_analyze(r->subquery, pstate);
 
 	/*
-	 * Check that we got something reasonable.	Some of these conditions
+	 * Check that we got something reasonable.	Most of these conditions
 	 * are probably impossible given restrictions of the grammar, but
 	 * check 'em anyway.
 	 */
 	if (length(parsetrees) != 1)
-		elog(ERROR, "Unexpected parse analysis result for subselect in FROM");
+		elog(ERROR, "unexpected parse analysis result for subquery in FROM");
 	query = (Query *) lfirst(parsetrees);
 	if (query == NULL || !IsA(query, Query))
-		elog(ERROR, "Unexpected parse analysis result for subselect in FROM");
+		elog(ERROR, "unexpected parse analysis result for subquery in FROM");
 
 	if (query->commandType != CMD_SELECT)
-		elog(ERROR, "Expected SELECT query from subselect in FROM");
-	if (query->resultRelation != 0 || query->into != NULL || query->isPortal)
-		elog(ERROR, "Subselect in FROM may not have SELECT INTO");
+		elog(ERROR, "expected SELECT query from subquery in FROM");
+	if (query->resultRelation != 0 || query->into != NULL)
+		ereport(ERROR,
+				(errcode(ERRCODE_SYNTAX_ERROR),
+				 errmsg("subquery in FROM may not have SELECT INTO")));
+
+	/*
+	 * The subquery cannot make use of any variables from FROM items
+	 * created earlier in the current query.  Per SQL92, the scope of a
+	 * FROM item does not include other FROM items.  Formerly we hacked
+	 * the namespace so that the other variables weren't even visible, but
+	 * it seems more useful to leave them visible and give a specific
+	 * error message.
+	 *
+	 * XXX this will need further work to support SQL99's LATERAL() feature,
+	 * wherein such references would indeed be legal.
+	 *
+	 * We can skip groveling through the subquery if there's not anything
+	 * visible in the current query.  Also note that outer references are
+	 * OK.
+	 */
+	if (pstate->p_namespace)
+	{
+		if (contain_vars_of_level((Node *) query, 1))
+			ereport(ERROR,
+					(errcode(ERRCODE_INVALID_COLUMN_REFERENCE),
+					 errmsg("subquery in FROM may not refer to other relations of same query level")));
+	}
 
 	/*
 	 * OK, build an RTE for the subquery.
@@ -458,7 +467,6 @@ transformRangeFunction(ParseState *pstate, RangeFunction *r)
 {
 	Node	   *funcexpr;
 	char	   *funcname;
-	List	   *save_namespace;
 	RangeTblEntry *rte;
 	RangeTblRef *rtr;
 
@@ -467,31 +475,26 @@ transformRangeFunction(ParseState *pstate, RangeFunction *r)
 	funcname = strVal(llast(((FuncCall *) r->funccallnode)->funcname));
 
 	/*
-	 * Transform the raw FuncCall node.  This is a bit tricky because we
-	 * don't want the function expression to be able to see any FROM items
-	 * already created in the current query (compare to
-	 * transformRangeSubselect). But it does need to be able to see any
-	 * further-up parent states. So, temporarily make the current query
-	 * level have an empty namespace. NOTE: this code is OK only because
-	 * the expression can't legally alter the namespace by causing
-	 * implicit relation refs to be added.
+	 * Transform the raw FuncCall node.
 	 */
-	save_namespace = pstate->p_namespace;
-	pstate->p_namespace = NIL;
-
 	funcexpr = transformExpr(pstate, r->funccallnode);
 
-	pstate->p_namespace = save_namespace;
-
 	/*
-	 * We still need to check that the function parameters don't refer to
-	 * any other rels.	That could happen despite our hack on the
-	 * namespace if fully-qualified names are used.  So, check there are
-	 * no local Var references in the transformed expression.  (Outer
-	 * references are OK, and are ignored here.)
+	 * The function parameters cannot make use of any variables from other
+	 * FROM items.	(Compare to transformRangeSubselect(); the coding is
+	 * different though because we didn't parse as a sub-select with its
+	 * own level of namespace.)
+	 *
+	 * XXX this will need further work to support SQL99's LATERAL() feature,
+	 * wherein such references would indeed be legal.
 	 */
-	if (pull_varnos(funcexpr) != NIL)
-		elog(ERROR, "FROM function expression may not refer to other relations of same query level");
+	if (pstate->p_namespace)
+	{
+		if (contain_vars_of_level(funcexpr, 0))
+			ereport(ERROR,
+					(errcode(ERRCODE_INVALID_COLUMN_REFERENCE),
+					 errmsg("function expression in FROM may not refer to other relations of same query level")));
+	}
 
 	/*
 	 * Disallow aggregate functions in the expression.	(No reason to
@@ -499,8 +502,10 @@ transformRangeFunction(ParseState *pstate, RangeFunction *r)
 	 */
 	if (pstate->p_hasAggs)
 	{
-		if (contain_agg_clause(funcexpr))
-			elog(ERROR, "cannot use aggregate function in FROM function expression");
+		if (checkExprHasAggs(funcexpr))
+			ereport(ERROR,
+					(errcode(ERRCODE_GROUPING_ERROR),
+					 errmsg("cannot use aggregate function in function expression in FROM")));
 	}
 
 	/*
@@ -515,17 +520,6 @@ transformRangeFunction(ParseState *pstate, RangeFunction *r)
 		CheckAttributeNamesTypes(tupdesc, RELKIND_COMPOSITE_TYPE);
 	}
 
-	/*
-	 * Insist we have a bare function call (explain.c is the only place
-	 * that depends on this, I think).	If this fails, it's probably
-	 * because transformExpr interpreted the function notation as a type
-	 * coercion.
-	 */
-	if (!funcexpr ||
-		!IsA(funcexpr, Expr) ||
-		((Expr *) funcexpr)->opType != FUNC_EXPR)
-		elog(ERROR, "Coercion function not allowed in FROM clause");
-
 	/*
 	 * OK, build an RTE for the function.
 	 */
@@ -634,7 +628,7 @@ transformFromClauseItem(ParseState *pstate, Node *n, List **containedRels)
 			leftrti = ((JoinExpr *) j->larg)->rtindex;
 		else
 		{
-			elog(ERROR, "transformFromClauseItem: unexpected subtree type");
+			elog(ERROR, "unrecognized node type: %d", (int) nodeTag(j->larg));
 			leftrti = 0;		/* keep compiler quiet */
 		}
 		rte = rt_fetch(leftrti, pstate->p_rtable);
@@ -646,7 +640,7 @@ transformFromClauseItem(ParseState *pstate, Node *n, List **containedRels)
 			rightrti = ((JoinExpr *) j->rarg)->rtindex;
 		else
 		{
-			elog(ERROR, "transformFromClauseItem: unexpected subtree type");
+			elog(ERROR, "unrecognized node type: %d", (int) nodeTag(j->rarg));
 			rightrti = 0;		/* keep compiler quiet */
 		}
 		rte = rt_fetch(rightrti, pstate->p_rtable);
@@ -729,7 +723,10 @@ transformFromClauseItem(ParseState *pstate, Node *n, List **containedRels)
 					char	   *res_colname = strVal(lfirst(col));
 
 					if (strcmp(res_colname, u_colname) == 0)
-						elog(ERROR, "USING column name \"%s\" appears more than once", u_colname);
+						ereport(ERROR,
+								(errcode(ERRCODE_DUPLICATE_COLUMN),
+								 errmsg("column name \"%s\" appears more than once in USING clause",
+										u_colname)));
 				}
 
 				/* Find it in left input */
@@ -741,14 +738,19 @@ transformFromClauseItem(ParseState *pstate, Node *n, List **containedRels)
 					if (strcmp(l_colname, u_colname) == 0)
 					{
 						if (l_index >= 0)
-							elog(ERROR, "Common column name \"%s\" appears more than once in left table", u_colname);
+							ereport(ERROR,
+									(errcode(ERRCODE_AMBIGUOUS_COLUMN),
+									 errmsg("common column name \"%s\" appears more than once in left table",
+											u_colname)));
 						l_index = ndx;
 					}
 					ndx++;
 				}
 				if (l_index < 0)
-					elog(ERROR, "JOIN/USING column \"%s\" not found in left table",
-						 u_colname);
+					ereport(ERROR,
+							(errcode(ERRCODE_UNDEFINED_COLUMN),
+							 errmsg("column \"%s\" specified in USING clause does not exist in left table",
+									u_colname)));
 
 				/* Find it in right input */
 				ndx = 0;
@@ -759,14 +761,19 @@ transformFromClauseItem(ParseState *pstate, Node *n, List **containedRels)
 					if (strcmp(r_colname, u_colname) == 0)
 					{
 						if (r_index >= 0)
-							elog(ERROR, "Common column name \"%s\" appears more than once in right table", u_colname);
+							ereport(ERROR,
+									(errcode(ERRCODE_AMBIGUOUS_COLUMN),
+									 errmsg("common column name \"%s\" appears more than once in right table",
+											u_colname)));
 						r_index = ndx;
 					}
 					ndx++;
 				}
 				if (r_index < 0)
-					elog(ERROR, "JOIN/USING column \"%s\" not found in right table",
-						 u_colname);
+					ereport(ERROR,
+							(errcode(ERRCODE_UNDEFINED_COLUMN),
+							 errmsg("column \"%s\" specified in USING clause does not exist in right table",
+									u_colname)));
 
 				l_colvar = nth(l_index, l_colvars);
 				l_usingvars = lappend(l_usingvars, l_colvar);
@@ -775,7 +782,8 @@ transformFromClauseItem(ParseState *pstate, Node *n, List **containedRels)
 
 				res_colnames = lappend(res_colnames, lfirst(ucol));
 				res_colvars = lappend(res_colvars,
-									  buildMergedJoinVar(j->jointype,
+									  buildMergedJoinVar(pstate,
+														 j->jointype,
 														 l_colvar,
 														 r_colvar));
 			}
@@ -814,8 +822,10 @@ transformFromClauseItem(ParseState *pstate, Node *n, List **containedRels)
 			if (j->alias->colnames != NIL)
 			{
 				if (length(j->alias->colnames) > length(res_colnames))
-					elog(ERROR, "Column alias list for \"%s\" has too many entries",
-						 j->alias->aliasname);
+					ereport(ERROR,
+							(errcode(ERRCODE_SYNTAX_ERROR),
+							 errmsg("column alias list for \"%s\" has too many entries",
+									j->alias->aliasname)));
 			}
 		}
 
@@ -841,10 +851,8 @@ transformFromClauseItem(ParseState *pstate, Node *n, List **containedRels)
 		return (Node *) j;
 	}
 	else
-		elog(ERROR, "transformFromClauseItem: unexpected node (internal error)"
-			 "\n\t%s", nodeToString(n));
-	return NULL;				/* can't get here, just keep compiler
-								 * quiet */
+		elog(ERROR, "unrecognized node type: %d", (int) nodeTag(n));
+	return NULL;				/* can't get here, keep compiler quiet */
 }
 
 /*
@@ -852,7 +860,8 @@ transformFromClauseItem(ParseState *pstate, Node *n, List **containedRels)
  *	  generate a suitable replacement expression for a merged join column
  */
 static Node *
-buildMergedJoinVar(JoinType jointype, Var *l_colvar, Var *r_colvar)
+buildMergedJoinVar(ParseState *pstate, JoinType jointype,
+				   Var *l_colvar, Var *r_colvar)
 {
 	Oid			outcoltype;
 	int32		outcoltypmod;
@@ -867,7 +876,7 @@ buildMergedJoinVar(JoinType jointype, Var *l_colvar, Var *r_colvar)
 	outcoltypmod = l_colvar->vartypmod;
 	if (outcoltype != r_colvar->vartype)
 	{
-		outcoltype = select_common_type(makeListi2(l_colvar->vartype,
+		outcoltype = select_common_type(makeListo2(l_colvar->vartype,
 												   r_colvar->vartype),
 										"JOIN/USING");
 		outcoltypmod = -1;		/* ie, unknown */
@@ -885,22 +894,22 @@ buildMergedJoinVar(JoinType jointype, Var *l_colvar, Var *r_colvar)
 	 * typmod is not same as input.
 	 */
 	if (l_colvar->vartype != outcoltype)
-		l_node = coerce_type((Node *) l_colvar, l_colvar->vartype,
+		l_node = coerce_type(pstate, (Node *) l_colvar, l_colvar->vartype,
 							 outcoltype,
 							 COERCION_IMPLICIT, COERCE_IMPLICIT_CAST);
 	else if (l_colvar->vartypmod != outcoltypmod)
-		l_node = (Node *) makeRelabelType((Node *) l_colvar,
+		l_node = (Node *) makeRelabelType((Expr *) l_colvar,
 										  outcoltype, outcoltypmod,
 										  COERCE_IMPLICIT_CAST);
 	else
 		l_node = (Node *) l_colvar;
 
 	if (r_colvar->vartype != outcoltype)
-		r_node = coerce_type((Node *) r_colvar, r_colvar->vartype,
+		r_node = coerce_type(pstate, (Node *) r_colvar, r_colvar->vartype,
 							 outcoltype,
 							 COERCION_IMPLICIT, COERCE_IMPLICIT_CAST);
 	else if (r_colvar->vartypmod != outcoltypmod)
-		r_node = (Node *) makeRelabelType((Node *) r_colvar,
+		r_node = (Node *) makeRelabelType((Expr *) r_colvar,
 										  outcoltype, outcoltypmod,
 										  COERCE_IMPLICIT_CAST);
 	else
@@ -937,23 +946,15 @@ buildMergedJoinVar(JoinType jointype, Var *l_colvar, Var *r_colvar)
 				 * Here we must build a COALESCE expression to ensure that
 				 * the join output is non-null if either input is.
 				 */
-				CaseExpr   *c = makeNode(CaseExpr);
-				CaseWhen   *w = makeNode(CaseWhen);
-				NullTest   *n = makeNode(NullTest);
+				CoalesceExpr *c = makeNode(CoalesceExpr);
 
-				n->arg = l_node;
-				n->nulltesttype = IS_NOT_NULL;
-				w->expr = (Node *) n;
-				w->result = l_node;
-				c->casetype = outcoltype;
-				c->args = makeList1(w);
-				c->defresult = r_node;
+				c->coalescetype = outcoltype;
+				c->args = makeList2(l_node, r_node);
 				res_node = (Node *) c;
 				break;
 			}
 		default:
-			elog(ERROR, "buildMergedJoinVar: unexpected jointype %d",
-				 (int) jointype);
+			elog(ERROR, "unrecognized join type: %d", (int) jointype);
 			res_node = NULL;	/* keep compiler quiet */
 			break;
 	}
@@ -964,10 +965,14 @@ buildMergedJoinVar(JoinType jointype, Var *l_colvar, Var *r_colvar)
 
 /*
  * transformWhereClause -
- *	  transforms the qualification and make sure it is of type Boolean
+ *	  Transform the qualification and make sure it is of type boolean.
+ *	  Used for WHERE and allied clauses.
+ *
+ * constructName does not affect the semantics, but is used in error messages
  */
 Node *
-transformWhereClause(ParseState *pstate, Node *clause)
+transformWhereClause(ParseState *pstate, Node *clause,
+					 const char *constructName)
 {
 	Node	   *qual;
 
@@ -976,7 +981,61 @@ transformWhereClause(ParseState *pstate, Node *clause)
 
 	qual = transformExpr(pstate, clause);
 
-	qual = coerce_to_boolean(qual, "WHERE");
+	qual = coerce_to_boolean(pstate, qual, constructName);
+
+	return qual;
+}
+
+
+/*
+ * transformLimitClause -
+ *	  Transform the expression and make sure it is of type integer.
+ *	  Used for LIMIT and allied clauses.
+ *
+ * constructName does not affect the semantics, but is used in error messages
+ */
+Node *
+transformLimitClause(ParseState *pstate, Node *clause,
+					 const char *constructName)
+{
+	Node	   *qual;
+
+	if (clause == NULL)
+		return NULL;
+
+	qual = transformExpr(pstate, clause);
+
+	qual = coerce_to_integer(pstate, qual, constructName);
+
+	/*
+	 * LIMIT can't refer to any vars or aggregates of the current query;
+	 * we don't allow subselects either (though that case would at least
+	 * be sensible)
+	 */
+	if (contain_vars_of_level(qual, 0))
+	{
+		ereport(ERROR,
+				(errcode(ERRCODE_INVALID_COLUMN_REFERENCE),
+		/* translator: %s is name of a SQL construct, eg LIMIT */
+				 errmsg("argument of %s must not contain variables",
+						constructName)));
+	}
+	if (checkExprHasAggs(qual))
+	{
+		ereport(ERROR,
+				(errcode(ERRCODE_GROUPING_ERROR),
+		/* translator: %s is name of a SQL construct, eg LIMIT */
+				 errmsg("argument of %s must not contain aggregates",
+						constructName)));
+	}
+	if (contain_subplans(qual))
+	{
+		ereport(ERROR,
+				(errcode(ERRCODE_FEATURE_NOT_SUPPORTED),
+		/* translator: %s is name of a SQL construct, eg LIMIT */
+				 errmsg("argument of %s must not contain subqueries",
+						constructName)));
+	}
 
 	return qual;
 }
@@ -1053,10 +1112,18 @@ findTargetlistEntry(ParseState *pstate, Node *node, List *tlist, int clause)
 			 * is a matching column.  If so, fall through to let
 			 * transformExpr() do the rest.  NOTE: if name could refer
 			 * ambiguously to more than one column name exposed by FROM,
-			 * colnameToVar will elog(ERROR).  That's just what we want
+			 * colNameToVar will ereport(ERROR).  That's just what we want
 			 * here.
+			 *
+			 * Small tweak for 7.4.3: ignore matches in upper query levels.
+			 * This effectively changes the search order for bare names to
+			 * (1) local FROM variables, (2) local targetlist aliases,
+			 * (3) outer FROM variables, whereas before it was (1) (3) (2).
+			 * SQL92 and SQL99 do not allow GROUPing BY an outer reference,
+			 * so this breaks no cases that are legal per spec, and it
+			 * seems a more self-consistent behavior.
 			 */
-			if (colnameToVar(pstate, name) != NULL)
+			if (colNameToVar(pstate, name, true) != NULL)
 				name = NULL;
 		}
 
@@ -1073,8 +1140,11 @@ findTargetlistEntry(ParseState *pstate, Node *node, List *tlist, int clause)
 					if (target_result != NULL)
 					{
 						if (!equal(target_result->expr, tle->expr))
-							elog(ERROR, "%s '%s' is ambiguous",
-								 clauseText[clause], name);
+							ereport(ERROR,
+									(errcode(ERRCODE_AMBIGUOUS_COLUMN),
+									 /* translator: first %s is name of a SQL construct, eg ORDER BY */
+									 errmsg("%s \"%s\" is ambiguous",
+											clauseText[clause], name)));
 					}
 					else
 						target_result = tle;
@@ -1092,7 +1162,11 @@ findTargetlistEntry(ParseState *pstate, Node *node, List *tlist, int clause)
 		int			target_pos;
 
 		if (!IsA(val, Integer))
-			elog(ERROR, "Non-integer constant in %s", clauseText[clause]);
+			ereport(ERROR,
+					(errcode(ERRCODE_SYNTAX_ERROR),
+			/* translator: %s is name of a SQL construct, eg ORDER BY */
+					 errmsg("non-integer constant in %s",
+							clauseText[clause])));
 		target_pos = intVal(val);
 		foreach(tl, tlist)
 		{
@@ -1105,8 +1179,11 @@ findTargetlistEntry(ParseState *pstate, Node *node, List *tlist, int clause)
 					return tle; /* return the unique match */
 			}
 		}
-		elog(ERROR, "%s position %d is not in target list",
-			 clauseText[clause], target_pos);
+		ereport(ERROR,
+				(errcode(ERRCODE_INVALID_COLUMN_REFERENCE),
+		/* translator: %s is name of a SQL construct, eg ORDER BY */
+				 errmsg("%s position %d is not in select list",
+						clauseText[clause], target_pos)));
 	}
 
 	/*
@@ -1141,11 +1218,11 @@ findTargetlistEntry(ParseState *pstate, Node *node, List *tlist, int clause)
 
 /*
  * transformGroupClause -
- *	  transform a Group By clause
- *
+ *	  transform a GROUP BY clause
  */
 List *
-transformGroupClause(ParseState *pstate, List *grouplist, List *targetlist)
+transformGroupClause(ParseState *pstate, List *grouplist,
+					 List *targetlist, List *sortClause)
 {
 	List	   *glist = NIL,
 			   *gl;
@@ -1153,21 +1230,55 @@ transformGroupClause(ParseState *pstate, List *grouplist, List *targetlist)
 	foreach(gl, grouplist)
 	{
 		TargetEntry *tle;
+		Oid			restype;
+		Oid			ordering_op;
+		GroupClause *grpcl;
 
 		tle = findTargetlistEntry(pstate, lfirst(gl),
 								  targetlist, GROUP_CLAUSE);
 
 		/* avoid making duplicate grouplist entries */
-		if (!targetIsInSortList(tle, glist))
+		if (targetIsInSortList(tle, glist))
+			continue;
+
+		/* if tlist item is an UNKNOWN literal, change it to TEXT */
+		restype = tle->resdom->restype;
+
+		if (restype == UNKNOWNOID)
 		{
-			GroupClause *grpcl = makeNode(GroupClause);
-
-			grpcl->tleSortGroupRef = assignSortGroupRef(tle, targetlist);
-
-			grpcl->sortop = any_ordering_op(tle->resdom->restype);
-
-			glist = lappend(glist, grpcl);
+			tle->expr = (Expr *) coerce_type(pstate, (Node *) tle->expr,
+											 restype, TEXTOID,
+											 COERCION_IMPLICIT,
+											 COERCE_IMPLICIT_CAST);
+			restype = tle->resdom->restype = TEXTOID;
+			tle->resdom->restypmod = -1;
 		}
+
+		/*
+		 * If the GROUP BY clause matches the ORDER BY clause, we want to
+		 * adopt the ordering operators from the latter rather than using
+		 * the default ops.  This allows "GROUP BY foo ORDER BY foo DESC"
+		 * to be done with only one sort step.	Note we are assuming that
+		 * any user-supplied ordering operator will bring equal values
+		 * together, which is all that GROUP BY needs.
+		 */
+		if (sortClause &&
+			((SortClause *) lfirst(sortClause))->tleSortGroupRef ==
+			tle->resdom->ressortgroupref)
+		{
+			ordering_op = ((SortClause *) lfirst(sortClause))->sortop;
+			sortClause = lnext(sortClause);
+		}
+		else
+		{
+			ordering_op = ordering_oper_opid(restype);
+			sortClause = NIL;	/* disregard ORDER BY once match fails */
+		}
+
+		grpcl = makeNode(GroupClause);
+		grpcl->tleSortGroupRef = assignSortGroupRef(tle, targetlist);
+		grpcl->sortop = ordering_op;
+		glist = lappend(glist, grpcl);
 	}
 
 	return glist;
@@ -1180,21 +1291,25 @@ transformGroupClause(ParseState *pstate, List *grouplist, List *targetlist)
 List *
 transformSortClause(ParseState *pstate,
 					List *orderlist,
-					List *targetlist)
+					List *targetlist,
+					bool resolveUnknown)
 {
 	List	   *sortlist = NIL;
 	List	   *olitem;
 
 	foreach(olitem, orderlist)
 	{
-		SortGroupBy *sortby = lfirst(olitem);
+		SortBy	   *sortby = lfirst(olitem);
 		TargetEntry *tle;
 
 		tle = findTargetlistEntry(pstate, sortby->node,
 								  targetlist, ORDER_CLAUSE);
 
-		sortlist = addTargetToSortList(tle, sortlist, targetlist,
-									   sortby->useOp);
+		sortlist = addTargetToSortList(pstate, tle,
+									   sortlist, targetlist,
+									   sortby->sortby_kind,
+									   sortby->useOp,
+									   resolveUnknown);
 	}
 
 	return sortlist;
@@ -1232,7 +1347,10 @@ transformDistinctClause(ParseState *pstate, List *distinctlist,
 		 * the user's ORDER BY spec alone, and just add additional sort
 		 * keys to it to ensure that all targetlist items get sorted.)
 		 */
-		*sortClause = addAllTargetsToSortList(*sortClause, targetlist);
+		*sortClause = addAllTargetsToSortList(pstate,
+											  *sortClause,
+											  targetlist,
+											  true);
 
 		/*
 		 * Now, DISTINCT list consists of all non-resjunk sortlist items.
@@ -1247,7 +1365,9 @@ transformDistinctClause(ParseState *pstate, List *distinctlist,
 			TargetEntry *tle = get_sortgroupclause_tle(scl, targetlist);
 
 			if (tle->resdom->resjunk)
-				elog(ERROR, "For SELECT DISTINCT, ORDER BY expressions must appear in target list");
+				ereport(ERROR,
+						(errcode(ERRCODE_INVALID_COLUMN_REFERENCE),
+						 errmsg("for SELECT DISTINCT, ORDER BY expressions must appear in select list")));
 			else
 				result = lappend(result, copyObject(scl));
 		}
@@ -1285,14 +1405,17 @@ transformDistinctClause(ParseState *pstate, List *distinctlist,
 				SortClause *scl = (SortClause *) lfirst(nextsortlist);
 
 				if (tle->resdom->ressortgroupref != scl->tleSortGroupRef)
-					elog(ERROR, "SELECT DISTINCT ON expressions must match initial ORDER BY expressions");
+					ereport(ERROR,
+							(errcode(ERRCODE_INVALID_COLUMN_REFERENCE),
+							 errmsg("SELECT DISTINCT ON expressions must match initial ORDER BY expressions")));
 				result = lappend(result, copyObject(scl));
 				nextsortlist = lnext(nextsortlist);
 			}
 			else
 			{
-				*sortClause = addTargetToSortList(tle, *sortClause,
-												  targetlist, NIL);
+				*sortClause = addTargetToSortList(pstate, tle,
+												  *sortClause, targetlist,
+												  SORTBY_ASC, NIL, true);
 
 				/*
 				 * Probably, the tle should always have been added at the
@@ -1308,8 +1431,8 @@ transformDistinctClause(ParseState *pstate, List *distinctlist,
 						break;
 					}
 				}
-				if (slitem == NIL)
-					elog(ERROR, "transformDistinctClause: failed to add DISTINCT ON clause to target list");
+				if (slitem == NIL)		/* should not happen */
+					elog(ERROR, "failed to add DISTINCT ON clause to target list");
 			}
 		}
 	}
@@ -1323,10 +1446,13 @@ transformDistinctClause(ParseState *pstate, List *distinctlist,
  *		ORDER BY list, adding the not-yet-sorted ones to the end of the list.
  *		This is typically used to help implement SELECT DISTINCT.
  *
+ * See addTargetToSortList for info about pstate and resolveUnknown inputs.
+ *
  * Returns the updated ORDER BY list.
  */
 List *
-addAllTargetsToSortList(List *sortlist, List *targetlist)
+addAllTargetsToSortList(ParseState *pstate, List *sortlist,
+						List *targetlist, bool resolveUnknown)
 {
 	List	   *i;
 
@@ -1335,7 +1461,10 @@ addAllTargetsToSortList(List *sortlist, List *targetlist)
 		TargetEntry *tle = (TargetEntry *) lfirst(i);
 
 		if (!tle->resdom->resjunk)
-			sortlist = addTargetToSortList(tle, sortlist, targetlist, NIL);
+			sortlist = addTargetToSortList(pstate, tle,
+										   sortlist, targetlist,
+										   SORTBY_ASC, NIL,
+										   resolveUnknown);
 	}
 	return sortlist;
 }
@@ -1344,28 +1473,59 @@ addAllTargetsToSortList(List *sortlist, List *targetlist)
  * addTargetToSortList
  *		If the given targetlist entry isn't already in the ORDER BY list,
  *		add it to the end of the list, using the sortop with given name
- *		or any available sort operator if opname == NIL.
+ *		or the default sort operator if opname == NIL.
+ *
+ * If resolveUnknown is TRUE, convert TLEs of type UNKNOWN to TEXT.  If not,
+ * do nothing (which implies the search for a sort operator will fail).
+ * pstate should be provided if resolveUnknown is TRUE, but can be NULL
+ * otherwise.
  *
  * Returns the updated ORDER BY list.
  */
-static List *
-addTargetToSortList(TargetEntry *tle, List *sortlist, List *targetlist,
-					List *opname)
+List *
+addTargetToSortList(ParseState *pstate, TargetEntry *tle,
+					List *sortlist, List *targetlist,
+					int sortby_kind, List *sortby_opname,
+					bool resolveUnknown)
 {
 	/* avoid making duplicate sortlist entries */
 	if (!targetIsInSortList(tle, sortlist))
 	{
 		SortClause *sortcl = makeNode(SortClause);
+		Oid			restype = tle->resdom->restype;
+
+		/* if tlist item is an UNKNOWN literal, change it to TEXT */
+		if (restype == UNKNOWNOID && resolveUnknown)
+		{
+			tle->expr = (Expr *) coerce_type(pstate, (Node *) tle->expr,
+											 restype, TEXTOID,
+											 COERCION_IMPLICIT,
+											 COERCE_IMPLICIT_CAST);
+			restype = tle->resdom->restype = TEXTOID;
+			tle->resdom->restypmod = -1;
+		}
 
 		sortcl->tleSortGroupRef = assignSortGroupRef(tle, targetlist);
 
-		if (opname)
-			sortcl->sortop = compatible_oper_opid(opname,
-												  tle->resdom->restype,
-												  tle->resdom->restype,
-												  false);
-		else
-			sortcl->sortop = any_ordering_op(tle->resdom->restype);
+		switch (sortby_kind)
+		{
+			case SORTBY_ASC:
+				sortcl->sortop = ordering_oper_opid(restype);
+				break;
+			case SORTBY_DESC:
+				sortcl->sortop = reverse_ordering_oper_opid(restype);
+				break;
+			case SORTBY_USING:
+				Assert(sortby_opname != NIL);
+				sortcl->sortop = compatible_oper_opid(sortby_opname,
+													  restype,
+													  restype,
+													  false);
+				break;
+			default:
+				elog(ERROR, "unrecognized sortby_kind: %d", sortby_kind);
+				break;
+		}
 
 		sortlist = lappend(sortlist, sortcl);
 	}
@@ -1409,7 +1569,7 @@ assignSortGroupRef(TargetEntry *tle, List *tlist)
  * reason we need this routine (and not just a quick test for nonzeroness
  * of ressortgroupref) is that a TLE might be in only one of the lists.
  */
-static bool
+bool
 targetIsInSortList(TargetEntry *tle, List *sortList)
 {
 	Index		ref = tle->resdom->ressortgroupref;
