@@ -34,7 +34,7 @@
  *
  *
  * IDENTIFICATION
- *		$Header: /cvsroot/pgsql-server/src/bin/pg_dump/pg_restore.c,v 1.43 2002/10/18 22:05:36 petere Exp $
+ *		$Header: /cvsroot/pgsql-server/src/bin/pg_dump/pg_restore.c,v 1.53 2003/10/20 21:05:12 tgl Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -55,8 +55,10 @@
 
 #include <unistd.h>
 
-#ifdef HAVE_GETOPT_H
-#include <getopt.h>
+#include "getopt_long.h"
+
+#ifndef HAVE_OPTRESET
+int			optreset;
 #endif
 
 #ifdef ENABLE_NLS
@@ -67,25 +69,21 @@
 
 /* Forward decls */
 static void usage(const char *progname);
-static char *_cleanupName(char *name);
-static char *_cleanupFuncName(char *name);
 
 typedef struct option optType;
 
-
 int
 main(int argc, char **argv)
 {
 	RestoreOptions *opts;
 	int			c;
 	Archive    *AH;
-	char	   *fileSpec = NULL;
+	char	   *inputFileSpec;
 	extern int	optind;
 	extern char *optarg;
 	static int	use_setsessauth = 0;
 	static int	disable_triggers = 0;
 
-#ifdef HAVE_GETOPT_LONG
 	struct option cmdopts[] = {
 		{"clean", 0, NULL, 'c'},
 		{"create", 0, NULL, 'C'},
@@ -124,8 +122,6 @@ main(int argc, char **argv)
 
 		{NULL, 0, NULL, 0}
 	};
-#endif   /* HAVE_GETOPT_LONG */
-
 
 #ifdef ENABLE_NLS
 	setlocale(LC_ALL, "");
@@ -135,10 +131,7 @@ main(int argc, char **argv)
 
 	opts = NewRestoreOptions();
 
-	if (!strrchr(argv[0], '/'))
-		progname = argv[0];
-	else
-		progname = strrchr(argv[0], '/') + 1;
+	progname = get_progname(argv[0]);
 
 	if (argc > 1)
 	{
@@ -154,11 +147,8 @@ main(int argc, char **argv)
 		}
 	}
 
-#ifdef HAVE_GETOPT_LONG
-	while ((c = getopt_long(argc, argv, "acCd:f:F:h:iI:lL:NoOp:P:rRsS:t:T:uU:vWxX:", cmdopts, NULL)) != -1)
-#else
-	while ((c = getopt(argc, argv, "acCd:f:F:h:iI:lL:NoOp:P:rRsS:t:T:uU:vWxX:")) != -1)
-#endif
+	while ((c = getopt_long(argc, argv, "acCd:f:F:h:iI:lL:NoOp:P:rRsS:t:T:uU:vWxX:",
+							cmdopts, NULL)) != -1)
 	{
 		switch (c)
 		{
@@ -173,11 +163,7 @@ main(int argc, char **argv)
 				opts->create = 1;
 				break;
 			case 'd':
-				if (strlen(optarg) != 0)
-				{
-					opts->dbname = strdup(optarg);
-					opts->useDB = 1;
-				}
+				opts->dbname = strdup(optarg);
 				break;
 			case 'f':			/* output file name */
 				opts->filename = strdup(optarg);
@@ -219,22 +205,22 @@ main(int argc, char **argv)
 				opts->rearrange = 1;
 				break;
 			case 'R':
-				opts->noReconnect = 1;
+				/* no-op, still accepted for backwards compatibility */
 				break;
 			case 'P':			/* Function */
 				opts->selTypes = 1;
 				opts->selFunction = 1;
-				opts->functionNames = _cleanupFuncName(optarg);
+				opts->functionNames = strdup(optarg);
 				break;
 			case 'I':			/* Index */
 				opts->selTypes = 1;
 				opts->selIndex = 1;
-				opts->indexNames = _cleanupName(optarg);
+				opts->indexNames = strdup(optarg);
 				break;
 			case 'T':			/* Trigger */
 				opts->selTypes = 1;
 				opts->selTrigger = 1;
-				opts->triggerNames = _cleanupName(optarg);
+				opts->triggerNames = strdup(optarg);
 				break;
 			case 's':			/* dump schema only */
 				opts->schemaOnly = 1;
@@ -246,7 +232,7 @@ main(int argc, char **argv)
 			case 't':			/* Dump data for this table only */
 				opts->selTypes = 1;
 				opts->selTable = 1;
-				opts->tableNames = _cleanupName(optarg);
+				opts->tableNames = strdup(optarg);
 				break;
 
 			case 'u':
@@ -272,7 +258,7 @@ main(int argc, char **argv)
 
 			case 'X':
 				if (strcmp(optarg, "use-set-session-authorization") == 0)
-					use_setsessauth = 1;
+					/* no-op, still allowed for compatibility */ ;
 				else if (strcmp(optarg, "disable-triggers") == 0)
 					disable_triggers = 1;
 				else
@@ -280,29 +266,40 @@ main(int argc, char **argv)
 					fprintf(stderr,
 							_("%s: invalid -X option -- %s\n"),
 							progname, optarg);
-					fprintf(stderr, _("Try '%s --help' for more information.\n"), progname);
+					fprintf(stderr, _("Try \"%s --help\" for more information.\n"), progname);
 					exit(1);
 				}
 				break;
 
-#ifdef HAVE_GETOPT_LONG
 				/* This covers the long options equivalent to -X xxx. */
 			case 0:
 				break;
-#endif
 
 			default:
-				fprintf(stderr, _("Try '%s --help' for more information.\n"), progname);
+				fprintf(stderr, _("Try \"%s --help\" for more information.\n"), progname);
 				exit(1);
 		}
 	}
 
 	if (optind < argc)
-		fileSpec = argv[optind];
+		inputFileSpec = argv[optind];
 	else
-		fileSpec = NULL;
+		inputFileSpec = NULL;
+
+	/* Should get at most one of -d and -f, else user is confused */
+	if (opts->dbname)
+	{
+		if (opts->filename)
+		{
+			fprintf(stderr, _("%s: cannot specify both -d and -f output\n"),
+					progname);
+			fprintf(stderr, _("Try \"%s --help\" for more information.\n"),
+					progname);
+			exit(1);
+		}
+		opts->useDB = 1;
+	}
 
-	opts->use_setsessauth = use_setsessauth;
 	opts->disable_triggers = disable_triggers;
 
 	if (opts->formatName)
@@ -333,7 +330,7 @@ main(int argc, char **argv)
 		}
 	}
 
-	AH = OpenArchive(fileSpec, opts->format);
+	AH = OpenArchive(inputFileSpec, opts->format);
 
 	/* Let the archiver know how noisy to be */
 	AH->verbose = opts->verbose;
@@ -347,19 +344,13 @@ main(int argc, char **argv)
 		SortTocByID(AH);
 
 	if (opts->rearrange)
+		SortTocByObjectType(AH);
+	else
 	{
-		MoveToStart(AH, "<Init>");
-		MoveToEnd(AH, "TABLE DATA");
-		MoveToEnd(AH, "BLOBS");
-		MoveToEnd(AH, "INDEX");
-		MoveToEnd(AH, "TRIGGER");
-		MoveToEnd(AH, "RULE");
-		MoveToEnd(AH, "SEQUENCE SET");
+		/* Database MUST be at start (see also SortTocByObjectType) */
+		MoveToStart(AH, "DATABASE");
 	}
 
-	/* Database MUST be at start */
-	MoveToStart(AH, "DATABASE");
-
 	if (opts->tocSummary)
 		PrintTOCSummary(AH, opts);
 	else
@@ -378,26 +369,16 @@ usage(const char *progname)
 	printf(_("  %s [OPTION]... [FILE]\n"), progname);
 
 	printf(_("\nGeneral options:\n"));
-#ifdef HAVE_GETOPT_LONG
 	printf(_("  -d, --dbname=NAME        output database name\n"));
 	printf(_("  -f, --file=FILENAME      output file name\n"));
 	printf(_("  -F, --format=c|t         specify backup file format\n"));
 	printf(_("  -i, --ignore-version     proceed even when server version mismatches\n"));
 	printf(_("  -l, --list               print summarized TOC of the archive\n"));
 	printf(_("  -v, --verbose            verbose mode\n"));
-#else /* not HAVE_GETOPT_LONG */
-	printf(_("  -d NAME                  output database name\n"));
-	printf(_("  -f FILENAME              output file name\n"));
-	printf(_("  -F c|t                   specify backup file format\n"));
-	printf(_("  -i                       proceed even when server version mismatches\n"));
-	printf(_("  -l                       print summarized TOC of the archive\n"));
-	printf(_("  -v                       verbose mode\n"));
-#endif /* not HAVE_GETOPT_LONG */
 	printf(_("  --help                   show this help, then exit\n"));
 	printf(_("  --version                output version information, then exit\n"));
 
 	printf(_("\nOptions controlling the output content:\n"));
-#ifdef HAVE_GETOPT_LONG
 	printf(_("  -a, --data-only          restore only the data, no schema\n"));
 	printf(_("  -c, --clean              clean (drop) schema prior to create\n"));
 	printf(_("  -C, --create             issue commands to create the database\n"));
@@ -406,136 +387,25 @@ usage(const char *progname)
 			 "                           output from this file\n"));
 	printf(_("  -N, --orig-order         restore in original dump order\n"));
 	printf(_("  -o, --oid-order          restore in OID order\n"));
-	printf(_("  -O, --no-owner           do not reconnect to database to match\n"
-			 "                           object owner\n"));
+	printf(_("  -O, --no-owner           do not output commands to set object ownership\n"));
 	printf(_("  -P, --function=NAME(args)\n"
 			 "                           restore named function\n"));
 	printf(_("  -r, --rearrange          rearrange output to put indexes etc. at end\n"));
-	printf(_("  -R, --no-reconnect       disallow ALL reconnections to the database\n"));
 	printf(_("  -s, --schema-only        restore only the schema, no data\n"));
 	printf(_("  -S, --superuser=NAME     specify the superuser user name to use for\n"
 			 "                           disabling triggers\n"));
 	printf(_("  -t, --table=NAME         restore named table\n"));
 	printf(_("  -T, --trigger=NAME       restore named trigger\n"));
 	printf(_("  -x, --no-privileges      skip restoration of access privileges (grant/revoke)\n"));
-	printf(_("  -X use-set-session-authorization, --use-set-session-authorization\n"
-			 "                           use SET SESSION AUTHORIZATION commands instead\n"
-			 "                           of reconnecting, if possible\n"));
 	printf(_("  -X disable-triggers, --disable-triggers\n"
 			 "                           disable triggers during data-only restore\n"));
-#else /* not HAVE_GETOPT_LONG */
-	printf(_("  -a                       restore only the data, no schema\n"));
-	printf(_("  -c                       clean (drop) schema prior to create\n"));
-	printf(_("  -C                       issue commands to create the database\n"));
-	printf(_("  -I NAME                  restore named index\n"));
-	printf(_("  -L FILENAME              use specified table of contents for ordering\n"
-			 "                           output from this file\n"));
-	printf(_("  -N                       restore in original dump order\n"));
-	printf(_("  -o                       restore in OID order\n"));
-	printf(_("  -O                       do not reconnect to database to match\n"
-			 "                           object owner\n"));
-	printf(_("  -P NAME(args)            restore named function\n"));
-	printf(_("  -r                       rearrange output to put indexes etc. at end\n"));
-	printf(_("  -R                       disallow ALL reconnections to the database\n"));
-	printf(_("  -s                       restore only the schema, no data\n"));
-	printf(_("  -S NAME                  specify the superuser user name to use for\n"
-			 "                           disabling triggers\n"));
-	printf(_("  -t NAME                  restore named table\n"));
-	printf(_("  -T NAME                  restore named trigger\n"));
-	printf(_("  -x                       skip restoration of access privileges (grant/revoke)\n"));
-	printf(_("  -X use-set-session-authorization\n"
-			 "                           use SET SESSION AUTHORIZATION commands instead\n"
-			 "                           of reconnecting, if possible\n"));
-	printf(_("  -X disable-triggers      disable triggers during data-only restore\n"));
-#endif /* not HAVE_GETOPT_LONG */
 
 	printf(_("\nConnection options:\n"));
-#ifdef HAVE_GETOPT_LONG
-	printf(_("  -h, --host=HOSTNAME      database server host name\n"));
+	printf(_("  -h, --host=HOSTNAME      database server host or socket directory\n"));
 	printf(_("  -p, --port=PORT          database server port number\n"));
 	printf(_("  -U, --username=NAME      connect as specified database user\n"));
 	printf(_("  -W, --password           force password prompt (should happen automatically)\n"));
-#else /* not HAVE_GETOPT_LONG */
-	printf(_("  -h HOSTNAME              database server host name\n"));
-	printf(_("  -p PORT                  database server port number\n"));
-	printf(_("  -U NAME                  connect as specified database user\n"));
-	printf(_("  -W                       force password prompt (should happen automatically)\n"));
-#endif /* not HAVE_GETOPT_LONG */
 
 	printf(_("\nIf no input file name is supplied, then standard input is used.\n\n"));
 	printf(_("Report bugs to <pgsql-bugs@postgresql.org>.\n"));
 }
-
-
-static char *
-_cleanupName(char *name)
-{
-	int			i;
-
-	if (!name || !name[0])
-		return NULL;
-
-	name = strdup(name);
-
-	if (name[0] == '"')
-	{
-		strcpy(name, &name[1]);
-		if (name[0] && *(name + strlen(name) - 1) == '"')
-			*(name + strlen(name) - 1) = '\0';
-	}
-	/* otherwise, convert table name to lowercase... */
-	else
-	{
-		for (i = 0; name[i]; i++)
-			if (isupper((unsigned char) name[i]))
-				name[i] = tolower((unsigned char) name[i]);
-	}
-	return name;
-}
-
-
-static char *
-_cleanupFuncName(char *name)
-{
-	int			i;
-	char	   *ch;
-
-	if (!name || !name[0])
-		return NULL;
-
-	name = strdup(name);
-
-	if (name[0] == '"')
-	{
-		strcpy(name, &name[1]);
-		if (strchr(name, '"') != NULL)
-			strcpy(strchr(name, '"'), strchr(name, '"') + 1);
-	}
-	/* otherwise, convert function name to lowercase... */
-	else
-	{
-		for (i = 0; name[i]; i++)
-			if (isupper((unsigned char) name[i]))
-				name[i] = tolower((unsigned char) name[i]);
-	}
-
-	/* strip out any space before paren */
-	ch = strchr(name, '(');
-	while (ch && ch > name && *(ch - 1) == ' ')
-	{
-		strcpy(ch - 1, ch);
-		ch--;
-	}
-
-	/*
-	 * Strip out spaces after commas in parameter list. We can't remove
-	 * all spaces because some types, like 'double precision' have spaces.
-	 */
-	if ((ch = strchr(name, '(')) != NULL)
-	{
-		while ((ch = strstr(ch, ", ")) != NULL)
-			strcpy(ch + 1, ch + 2);
-	}
-
-	return name;
-}
